package com.xefer.rebel;

import java.io.*;
import java.lang.*;
import java.security.*;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The main servlet that coordinates the REST interface
 * of the XBEL elements.
 *
 */
 
public class XBELServlet extends HttpServlet
{

  /** maximum size of the entity allowed for PUT or POST */
  int    maxsize;
  /** maximum size of the XBEL document allowed */
  int    maxfile;

  /**
   * Initialize the servlet and get application parameters
   *
   * @return a value of type 'String' with information about the servlet
   */
   
  public void init() {
  
    ServletContext context = getServletContext();
  
    maxsize = Integer.parseInt(context.getInitParameter("max-entity"));
    maxfile = Integer.parseInt(context.getInitParameter("max-size"));
    
  }

  // PUT PUT PUT
  
  /**
   *
   * The servlet is handling the PUT request
   *
   */
   
  public void doPut(HttpServletRequest req, HttpServletResponse resp)
    throws ServletException, IOException
  {
    int     length    = req.getContentLength();
    String  path      = req.getPathInfo();
    String  fullpath  = (String) req.getAttribute("com.xefer.rebel.xbel");

    if(length > maxsize) {

      throw new ResourceException(HttpServletResponse.SC_REQUEST_ENTITY_TOO_LARGE, "Maximum entity size allowed is " + maxsize + "bytes");

    }

    // stream of posted entity
    InputStream istream = req.getInputStream();

    // instantiate the current XBEL document
    XBELDocument document = new XBELDocument(fullpath);

    XBELElement entity   = XBELElementFactory.newElement(istream);
    XBELElement resource = document.getElement(path, false);

    //
    // if resource ins't null, replace it, otherwise
    // generate a new resource
    //

    if (resource != null) {

      //
      // put the element to the current resource the write out
      //

      document.putElement(resource, entity);
      document.write(getServletContext(), fullpath);

      // 
      // no content to return
      //   

      resp.setStatus(HttpServletResponse.SC_NO_CONTENT);

    }
    else {

      resource = document.getElement(path, true);

      //
      // put the element to the current resource the write out
      //

      document.putElement(resource, entity);
      document.write(getServletContext(), fullpath);

      // 
      // respond that resource has been created at the requested url
      //   

      resp.setStatus(HttpServletResponse.SC_CREATED);
      resp.addHeader("Location", req.getRequestURL().toString());

    }

  }

  // DELETE DELETE DELETE
  
  /**
   *
   * The servlet is handling the DELETE request
   *
   */
   
  public void doDelete(HttpServletRequest req, HttpServletResponse resp)
    throws ServletException, IOException
  {

    String  path      = req.getPathInfo();
    String  fullpath  = (String) req.getAttribute("com.xefer.rebel.xbel");

    //
    // create XBELDocument from path
    //

    XBELDocument document = new XBELDocument(fullpath);

    // extract the resource node based on the url path

    XBELElement element = document.getElement(path);

    //
    // delete the named resource and write out
    //

    document.deleteElement(element);
    document.write(getServletContext(), fullpath);

    // 
    // no content to return
    // 

    resp.setStatus(HttpServletResponse.SC_NO_CONTENT);

  }


  // POST POST POST
   
  /**
   *   
   * The servlet is handling the POST request
   *
   */
   
  public void doPost(HttpServletRequest req, HttpServletResponse resp)
    throws ServletException, IOException
  {
    
    int     length    = req.getContentLength();
    String  path      = req.getPathInfo();
    String  fullpath  = (String) req.getAttribute("com.xefer.rebel.xbel");

    StringBuffer url  = req.getRequestURL();

    if(length > maxsize) {

      throw new ResourceException(HttpServletResponse.SC_REQUEST_ENTITY_TOO_LARGE, "Maximum entity size allowed is " + maxsize + "bytes");

    }

    // stream of posted entity
    InputStream istream = req.getInputStream();

    // instantiate the current XBEL document
    XBELDocument document = new XBELDocument(fullpath);

    XBELElement superordinate = document.getElement(path);      
    XBELElement subordinate   = XBELElementFactory.newElement(istream);

    //
    // post the element then write out the new document
    //

    document.postElement(superordinate, subordinate);
    document.write(getServletContext(), fullpath);

    //
    // show location of created resource, if any
    //

    if(subordinate.isLocatable()) {
      resp.setStatus(HttpServletResponse.SC_CREATED);
      resp.addHeader("Location", subordinate.getLocationUrl(url));
    }
    else {
      resp.setStatus(HttpServletResponse.SC_NO_CONTENT);
    }

  }


  // GET GET GET
  
  /**
   *
   *
   * The servlet is handling the GET request
   *
   *
   *
   */
   
  public void doGet(HttpServletRequest req, HttpServletResponse resp)
    throws ServletException, IOException
  {

    String  path      = req.getPathInfo();
    String  fullpath  = (String) req.getAttribute("com.xefer.rebel.xbel");

    XBELDocument document = new XBELDocument(fullpath);

    // extract the resource node based on the url path

    XBELElement element = document.getElement(path);

    //
    // write out resource
    //

    resp.setContentType("text/xml");

    element.write(getServletContext(), resp.getOutputStream());
    
  }

}
